/* getinfo.c -- Get and print information about a file.
 *
 * Ripped from my Comp. Graphics term project.
 */

#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/X.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>
#include <dirent.h>
#include <xpm.h>
#include "defines.h"
#include "helpsys.h"

/* XPM */
static char * infobig_xpm[] = {
"40 30 2 1",
" 	c #BE00BE00BE00",
".	c #000000000000",
"                ........                ",
"              ..        ..              ",
"            ..            ..            ",
"           .                .           ",
"          .                  .          ",
"         .                    .         ",
"        .         ....         .        ",
"       .          ....          .       ",
"       .          ....          .       ",
"      .                          .      ",
"      .                          .      ",
"     .           .....            .     ",
"     .           .....            .     ",
"     .            ....            .     ",
"     .            ....            .     ",
"     .            ....            .     ",
"     .            ....            .     ",
"     .            ....            .     ",
"     .            ....            .     ",
"      .           ....           .      ",
"      .           ....           .      ",
"       .          ....          .       ",
"       .        ........        .       ",
"        .       ........       .        ",
"         .                    .         ",
"          .                  .          ",
"           .                .           ",
"            ..            ..            ",
"              ..        ..              ",
"                ........                "};

/* Dialog shell needs a parent */
extern Widget top_level;

/* We'll need gloabl access to these */
Pixmap infopix;
Widget uperms, uperms_label, uperms_read, uperms_write, uperms_execute;
Widget gperms, gperms_label, gperms_read, gperms_write, gperms_execute;
Widget aperms, aperms_label, aperms_read, aperms_write, aperms_execute;

/* Destroy the info popup when finished */
void InfoDone(Widget w, XtPointer client_data, XtPointer call_data) 
{ XtDestroyWidget((Widget)client_data); }

/* Create popup dialog to describe file.  Don't forget to read file info */
void display_info(char *thisitem)
{
  Widget info_shell, info_w, name_w, size_w, owner_w, flags_w, ok_button_w,
  info_label_w, which_w, help_button_w;
  XmString name_s, owner_s, flags_s, ok_s, help_s, xlabelstr;
  char *name_char, size_char[20], owner_char[180], flag_char[20];
  struct stat buf;
  struct passwd *pass;
  Arg args[8];
  Display *mydisplay;
  XpmAttributes attribs;
  long my_uid, my_gid, x, wid;
  int m = 0;

  name_char=(char *)malloc(sizeof(char)*90);
  /* Get file info using stat and getpwuid */
  stat(thisitem, &buf);
  pass = getpwuid(buf.st_uid);
  my_uid = getuid();
  my_gid = getgid();
  m = S_ISDIR(buf.st_mode);

  /* Create the popup.  Print all info using labels, pushbutton to close */
  XtSetArg(args[0], XmNtitle, "Information");
  XtSetArg(args[1], XmNwidth, 300);
  info_shell=XtCreatePopupShell("info_popup", vendorShellWidgetClass, 
				top_level, args, 2);
  info_w=XtVaCreateManagedWidget("info_form", xmFormWidgetClass,
				 info_shell, NULL);

  /* Create (from XPM data above) and display the information icon */
  if (infopix == NULL) {
    mydisplay=XtDisplay(top_level);
    attribs.closeness=40000;
    attribs.valuemask=XpmSize | XpmCloseness;
    XpmCreatePixmapFromData(mydisplay, DefaultRootWindow(mydisplay),
			    infobig_xpm, &infopix, NULL, &attribs);  
  }
  info_label_w=XtVaCreateManagedWidget("info_pix_w",xmLabelWidgetClass,info_w,
				       XmNleftAttachment, XmATTACH_FORM,
				       XmNtopAttachment, XmATTACH_FORM,
				       XmNlabelType, XmPIXMAP,
				       XmNlabelPixmap, infopix,
				       XmNlabelInsensitivePixmap, infopix,
				       NULL);

  /* Display the name of the file, tell size or if it is a directory */
  if (!m) {
    sprintf(name_char, "%s\nSize: %ld bytes", thisitem, (long)buf.st_size);
    name_s=XmStringCreateLtoR(name_char, XmSTRING_DEFAULT_CHARSET);
  }
  else {
    sprintf(name_char, "%s\n[Directory]", thisitem);
    name_s=XmStringCreateLtoR(name_char, XmSTRING_DEFAULT_CHARSET);
  }
  name_w=XtVaCreateManagedWidget("name_w", xmLabelWidgetClass, info_w,
				 XmNlabelString, name_s,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, info_label_w,
				 XmNrightAttachment, XmATTACH_FORM,
				 XmNtopAttachment, XmATTACH_FORM,
				 XmNheight, 30,
				 NULL);

  /* Tell about the owner */
  if (pass != NULL)
    sprintf(owner_char, "Owner: %s (%s)\nOwner's UID: %d\nOwner's Group ID: %d", 
	    pass->pw_gecos, pass->pw_name, buf.st_uid, buf.st_gid);
  else
    sprintf(owner_char, "Owner's UID %d - Unknown UID!\nOwner's Group ID: %d",
	    buf.st_uid, buf.st_gid);
  owner_s=XmStringCreateLtoR(owner_char, XmSTRING_DEFAULT_CHARSET);
  owner_w=XtVaCreateManagedWidget("owner_w", xmLabelWidgetClass, info_w,
				 XmNlabelString, owner_s,
				 XmNleftAttachment, XmATTACH_FORM,
				 XmNrightAttachment, XmATTACH_FORM,
				 XmNtopAttachment, XmATTACH_WIDGET,
				 XmNtopWidget, name_w,
				 NULL);

  /* Describe the permissions on the file/directory */
  sprintf(flag_char, "Permissions: %od", buf.st_mode);
  flags_s=XmStringCreateSimple(flag_char);
  flags_w=XtVaCreateManagedWidget("flags_w", xmLabelWidgetClass, info_w,
				 XmNlabelString, flags_s,
				 XmNleftAttachment, XmATTACH_FORM,
				 XmNrightAttachment, XmATTACH_FORM,
				 XmNtopAttachment, XmATTACH_WIDGET,
				 XmNtopWidget, owner_w,
				 NULL);

  /* Display checkboxes to allow changing permissions. */
  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNtopWidget, flags_w);
  uperms=XmCreateRowColumn(info_w, "info_rc1", args, 4);
  uperms_label=XmCreateLabel(uperms, "User", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_read=(Widget)XmCreateToggleButton(uperms, "read", args, 1);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_write=(Widget)XmCreateToggleButton(uperms, "write", args, 1);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  uperms_execute=(Widget)XmCreateToggleButton(uperms, "execute", args, 1);
  if (buf.st_mode & 256)
    XtVaSetValues(uperms_read, XmNset, True, NULL);
  if (buf.st_mode & 128)
    XtVaSetValues(uperms_write, XmNset, True, NULL);
  if (buf.st_mode & 64)
    XtVaSetValues(uperms_execute, XmNset, True, NULL);
  XtManageChild(uperms);
  XtManageChild(uperms_label);
  XtManageChild(uperms_read);
  XtManageChild(uperms_write);
  XtManageChild(uperms_execute);

  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNleftWidget, uperms);
  XtSetArg(args[3], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[4], XmNtopWidget, flags_w);
  gperms=XmCreateRowColumn(info_w, "info_rc2", args, 5);
  gperms_label=XmCreateLabel(gperms, "Group", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_read=(Widget)XmCreateToggleButton(gperms, "read", args, 1);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_write=(Widget)XmCreateToggleButton(gperms, "write", args, 1);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  gperms_execute=(Widget)XmCreateToggleButton(gperms, "execute", args, 1);
  if (buf.st_mode & 32)
    XtVaSetValues(gperms_read, XmNset, True, NULL);
  if (buf.st_mode & 16)
    XtVaSetValues(gperms_write, XmNset, True, NULL);
  if (buf.st_mode & 8)
    XtVaSetValues(gperms_execute, XmNset, True, NULL);
  XtManageChild(gperms);
  XtManageChild(gperms_label);
  XtManageChild(gperms_read);
  XtManageChild(gperms_write);
  XtManageChild(gperms_execute);

  XtSetArg(args[0], XmNorientation, XmVERTICAL);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNleftWidget, gperms);
  XtSetArg(args[3], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[4], XmNtopWidget, flags_w);
  XtSetArg(args[5], XmNrightAttachment, XmATTACH_FORM);
  aperms=XmCreateRowColumn(info_w, "info_rc3", args, 6);
  aperms_label=XmCreateLabel(aperms, "Others", NULL, 0);
  xlabelstr=XmStringCreateSimple("Read");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_read=(Widget)XmCreateToggleButton(aperms, "read", args, 1);
  xlabelstr=XmStringCreateSimple("Write");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_write=(Widget)XmCreateToggleButton(aperms, "write", args, 1);
  xlabelstr=XmStringCreateSimple("Execute");
  XtSetArg(args[0], XmNlabelString, xlabelstr);
  aperms_execute=(Widget)XmCreateToggleButton(aperms, "execute", args, 1);
  if (buf.st_mode & 4)
    XtVaSetValues(aperms_read, XmNset, True, NULL);
  if (buf.st_mode & 2)
    XtVaSetValues(aperms_write, XmNset, True, NULL);
  if (buf.st_mode & 1)
    XtVaSetValues(aperms_execute, XmNset, True, NULL);
  XtManageChild(aperms);
  XtManageChild(aperms_label);
  XtManageChild(aperms_read);
  XtManageChild(aperms_write);
  XtManageChild(aperms_execute);

  x=0;
  if (my_uid != 0) {
    if (buf.st_uid != my_uid) {
      XtSetSensitive(uperms, False);
      XtSetSensitive(gperms, False);
    }
    else
      x++;
    if (x == 0)
      XtSetSensitive(aperms, False);
  }

  /* Close the info dialog. */
  ok_s=XmStringCreateSimple("OK");
  help_s=XmStringCreateSimple("Help");
  XtVaGetValues(info_w, XmNwidth, &wid, NULL);
  ok_button_w=XtVaCreateManagedWidget("size_w", xmPushButtonWidgetClass,
				      info_w, XmNlabelType, XmSTRING,
				      XmNlabelString, ok_s,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, uperms,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNwidth, 137,
				      NULL);
  help_button_w=XtVaCreateManagedWidget("size_w", xmPushButtonWidgetClass,
					info_w, XmNlabelType, XmSTRING,
					XmNlabelString, help_s,
					XmNleftAttachment, XmATTACH_WIDGET,
					XmNleftWidget, ok_button_w,
					XmNrightAttachment, XmATTACH_FORM,
					XmNtopAttachment, XmATTACH_WIDGET,
					XmNtopWidget, aperms,
					XmNbottomAttachment, XmATTACH_FORM,
					NULL);

  /* Press the ok button when finished reading. */
  XtAddCallback(ok_button_w, XmNactivateCallback, InfoDone, info_shell);
  /* Press the Help button to get help. */
  XtAddCallback(help_button_w, XmNactivateCallback, HelpTopic,
	       (XtPointer)FILE_HELP_INFO);

  /* Popup the shell. */
  XtPopup(info_shell, XtGrabExclusive);

  XmStringFree(name_s);
  XmStringFree(owner_s);
  XmStringFree(flags_s);
  XmStringFree(ok_s);
  XmStringFree(help_s);
  free(name_char);
}

