/* main.c -- Main function for Bryan's NEW file manager.
 *
 * Last revised:
 * Written by Bryan Herger
 *
 * Distributed under GNU Public License; see LICENSE for details
 */

#include <Xm/Xm.h>
#include <Xm/Container.h>
#include <Xm/IconG.h>
#include <Xm/ScrolledW.h>
#include <Xm/Form.h>
#include <Xm/TextF.h>

#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "findfile.h"
#include "getdir.h"
#include "openfile.h"
#include "menubars.h"
#include "filearea.h"
#include "selection.h"
#include "permission.h"
#include "smbconfig.h"

/* X stuff -- application context and top level shell */
/* Other modules need access to top_level and fm_container */
XtAppContext app;
Widget top_level, fm_container, fm_scroll1, fm_textfield;

/* User's home directory */
char *home_dir;

/* User's UID and GID (Are you root?) */
uid_t local_uid;
gid_t local_gid;

/* Default resources -- no more app-defaults file */
char * fallback_resources[] = {
"XmBryanFM*fontList: -*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*",
"XmBryanFM*SMB_page*fontList: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*",
"XmBryanFM*ghelp*fontList: -*-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*",
"XmBryanFM*fm_textfield*background: white",
"XmBryanFM*fm_textfield*foreground: black",
"XmBryanFM*fm_container*background: white",
"XmBryanFM*fm_container*foreground: black",
"XmBryanFM*source_tf*background: white",
"XmBryanFM*dest_tf*background: white",
"XmBryanFM*smb_g_t1*background: white",
"XmBryanFM*smb_add*background: white",
"XmBryanFM*smb_list*background: white",
"XmBryanFM*goto_tf*background: white",
"XmBryanFM*dir_c_tf*background: white",
"XmBryanFM*find_tf*background: white",
"XmBryanFM*f_slist*background: white",
"XmBryanFM*ghelp*background: white",
NULL};

void main(int argc, char *argv[])
{
  int x;
  Widget fm_main_window, fm_menubar;
  Dimension sx1, sy1, x1, y1;
  Arg args[10];

  /* Learn some stuff about the user */
  home_dir = (char *)getenv("HOME");
  local_uid = getuid();
  local_gid = getgid();

  /* Create the application and main window things. */
  top_level = XtVaAppInitialize(&app, "XmBryanFM", NULL, 0, &argc, argv,
				fallback_resources, NULL);

  /* This block of code creates the main application window.  I will eventually
   * move this to an external function.  Eventually... */
  XtSetArg(args[0], XmNwidth, 600);
  XtSetArg(args[1], XmNheight, 450);
  fm_main_window = XmCreateMainWindow(top_level, "fm_main_window",
				      args, 2);
  XtManageChild(fm_main_window);

  fm_menubar = CreateMenuBar(fm_main_window);

  XtSetArg(args[0], XmNscrollingPolicy, XmAUTOMATIC);
  XtSetArg(args[1], XmNresizable, True);
  fm_scroll1 = XmCreateScrolledWindow(fm_main_window, "fm_scroll1",
				      args, 2);
  XtManageChild(fm_scroll1);

  fm_textfield = XmCreateTextField(fm_main_window, "fm_textfield", NULL, 0);
  XtManageChild(fm_textfield);
  XtAddCallback(fm_textfield, XmNactivateCallback, fm_openitem, (XtPointer)10);

  /* Does anybody out there know a better way to do this?  I don't have
   * printed docs, and the online stuff is kinda sketchy on XmContainer
   * parameters.  Ideally, you would never see or need horizontal scrolls
   * except for really long file names (i.e., everything in three/four
   * columns) */
  XtSetArg(args[0], XmNlayoutType, XmSPATIAL);
  XtSetArg(args[1], XmNselectionPolicy, XmSINGLE_SELECT);
  XtSetArg(args[2], XmNwidth, 600);
  XtSetArg(args[3], XmNheight, 450);
  /*  XtSetArg(args[2], XmNspatialStyle, XmGRID);
  XtSetArg(args[3], XmNspatialResizeModel, XmGROW_MINOR);
  XtSetArg(args[4], XmNspatialIncludeModel, XmAPPEND); */
  fm_container = XmCreateContainer(fm_scroll1, "fm_container", args, 4);
  XtManageChild(fm_container);
  XtAddCallback(fm_container, XmNselectionCallback, fm_container_select, NULL);
  XtAddCallback(fm_container, XmNdefaultActionCallback, fm_openitem, NULL);
  XtAddCallback(fm_container, XmNdestinationCallback, FindFileTransfer, NULL);
  XtVaGetValues(fm_container, XmNlayoutDirection, &x, NULL);

  XmMainWindowSetAreas(fm_main_window, fm_menubar, fm_textfield, NULL, NULL,
		       fm_scroll1);

  files = malloc(sizeof(struct filedata) * nfiles);
  
  GetUserInfo();
  load_dir(home_dir);
  DrawFileArea();

  /* Learn some stuff about the system */
  scan_for_smb();

  XtRealizeWidget(top_level);

  XtAddCallback(XtParent(fm_container), XmNresizeCallback, 
		ContainerResizeCB, NULL);

  XtAppMainLoop(app);
}





